from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from colorama import init, Fore, Style, Back
from selenium.webdriver.support import expected_conditions as EC
import sys
import logging
from selenium.webdriver.common.keys import Keys
from arguments import verbose, erreur
from arguments import *

#clique sur un élement avec un chemin indiqué
def click(driver, liste):
    verbose(f"Execution de la fonction click : {liste}")

    if len(liste) < 2 or (liste[1] == "wait" and len(liste) < 4):
        raise ValueError("La liste fournie est invalide. Assurez-vous qu'elle contient tous les éléments nécessaires.")

    chemin = None

    try:
        if liste[1] == "wait":
            chemin = liste[3]
            verbose(f"Clic sur élément : {chemin}")
            if chemin.startswith("/"):
                click = WebDriverWait(driver, liste[2]).until(EC.element_to_be_clickable((By.XPATH, chemin)))
            elif chemin.startswith("#"):
                click = WebDriverWait(driver, liste[2]).until(EC.element_to_be_clickable((By.ID, chemin[1:])))
            elif chemin.startswith("."):
                click = WebDriverWait(driver, liste[2]).until(EC.element_to_be_clickable((By.CSS_SELECTOR, chemin)))
            else:
                raise ValueError(f"Type de sélecteur inconnu pour : {chemin}")
            click.click()
        else:
            chemin = liste[1]
            verbose(Fore.CYAN + f"Clic sur élément : {chemin}")
            if chemin.startswith("/"):
                driver.find_element(By.XPATH, chemin).click()
            elif chemin.startswith("#"):
                driver.find_element(By.ID, chemin[1:]).click()
            elif chemin.startswith("."):
                driver.find_element(By.CSS_SELECTOR, chemin).click()
            else:
                raise ValueError(f"Type de sélecteur inconnu pour : {chemin}")

        verbose(f"Élément cliqué : {chemin}")
    except Exception as e:
        print(Fore.RED + f"Erreur critique lors du clic sur l'élément {chemin} : {e}")
        raise  

#écrit dans un élément avec un chemin indiqué
def write(driver, liste):
    if len(liste) < 3:
        raise ValueError("La liste fournie est invalide. Elle doit contenir au moins trois éléments : [commande, chemin, texte].")

    chemin = liste[1]
    texte = liste[2]

    try:
        verbose(f"Écriture dans l'élément {chemin} avec le texte : {texte}")

        if chemin.startswith("/"):
            element = driver.find_element(By.XPATH, chemin)
        elif chemin.startswith("#"):
            element = driver.find_element(By.ID, chemin[1:])
        elif chemin.startswith("."):
            element = driver.find_element(By.CSS_SELECTOR, chemin)
        else:
            raise ValueError(f"Type de sélecteur inconnu pour : {chemin}")

        if not element.is_enabled():
            raise ValueError(f"L'élément {chemin} n'est pas modifiable ou désactivé.")

        element.send_keys(texte)

        verbose(f"Texte écrit : {texte} dans l'élément {chemin}")
    except Exception as e:
        verboseRed(f"Erreur critique lors de l'écriture dans l'élément {chemin} : {e}")
        raise e

#néttoie puis écrit dans un élément avec un chemin indiqué
def CWrite(driver, liste):
    # Validation des entrées
    if len(liste) < 3:
        raise ValueError("La liste fournie est invalide. Elle doit contenir au moins trois éléments : [commande, chemin, texte].")

    chemin = liste[1]
    texte = liste[2]

    try:
        verbose(f"Écriture dans l'élément {chemin} avec le texte : {texte}")

        if chemin.startswith("/"):
            element = driver.find_element(By.XPATH, chemin)
        elif chemin.startswith("#"):
            element = driver.find_element(By.ID, chemin[1:])
        elif chemin.startswith("."):
            element = driver.find_element(By.CSS_SELECTOR, chemin)
        else:
            raise ValueError(f"Type de sélecteur inconnu pour : {chemin}")

        if not element.is_enabled():
            raise ValueError(f"L'élément {chemin} n'est pas modifiable ou désactivé.")

        element.clear()
        element.send_keys(texte)

        verbose(f"Texte écrit : {texte} dans l'élément {chemin}")
    except Exception as e:
        verboseRed(f"Erreur critique lors de l'écriture dans l'élément {chemin} : {e}")
        raise e

#écrit dans un élément avec un  chemin indiqué puis appius sur entrée
def writeEnter(driver, liste):
    # Validation des entrées
    if len(liste) < 3:
        raise ValueError("La liste fournie est invalide. Elle doit contenir au moins trois éléments : [commande, chemin, texte].")

    chemin = liste[1]
    texte = liste[2]

    try:
        verbose(f"Écriture dans l'élément {chemin} avec le texte : {texte}")

        if chemin.startswith("/"):
            element = driver.find_element(By.XPATH, chemin)
        elif chemin.startswith("#"):
            element = driver.find_element(By.ID, chemin[1:])
        elif chemin.startswith("."):
            element = driver.find_element(By.CSS_SELECTOR, chemin)
        else:
            raise ValueError(f"Type de sélecteur inconnu pour : {chemin}")

        if not element.is_enabled():
            raise ValueError(f"L'élément {chemin} n'est pas modifiable ou désactivé.")

        element.clear()
        element.send_keys(texte)
        element.send_keys(Keys.ENTER)

        verbose(f"Texte écrit : {texte} dans l'élément {chemin}")
    except Exception as e:
        erreur(f"Erreur critique lors de l'écriture dans l'élément {chemin} : {e}")
        raise  

#trouve un élément à partr de son texte
def find_element_by_type(driver, liste, element_type, identifier):
    try:
        if isinstance(identifier, str):
            identifier = [identifier]

        identifier_str = ' '.join(identifier)
        wait_time = int(liste[2]) if liste[1] == "wait" else None


        element_xpaths = {
            "button": f"//button[contains(text(), '{identifier_str}') or "
                      f"contains(@title, '{identifier_str}')]",
            "input": f"//input[contains(@id, '{identifier_str}') or "
                     f"contains(@name, '{identifier_str}') or "
                     f"contains(@placeholder, '{identifier_str}') or "
                     f"contains(@value, '{identifier_str}') or "
                     f"contains(@title, '{identifier_str}')]",
            "select": f"//select[contains(@name, '{identifier_str}')]",
            "link": f"//a[contains(text(), '{identifier_str}')]",
            "p": f"//p[contains(text(), '{identifier_str}')]",
            "span": f"//span[contains(text(), '{identifier_str}')]"
        }

        if element_type not in element_xpaths:
            erreur(f"Type d'élément inconnu : {element_type}")
            sys.exit()
            return None

        xpath = element_xpaths[element_type]

        if wait_time:
            element = WebDriverWait(driver, wait_time).until(
                EC.element_to_be_clickable((By.XPATH, xpath))
            )
        else:
            element = driver.find_element(By.XPATH, xpath)

        return element

    except Exception as e:
        logging.error(Fore.RED + f"Erreur lors de la recherche de l'élément {element_type}: {e}")
        verbose(f"Erreur lors de la recherche de l'élément {element_type} : {e}")
        raise e
        sys.exit()
        return None