from selenium import webdriver
from colorama import init, Fore, Style, Back
import pyodbc
import sys
from interpreteur import execute
from utils import *
from arguments import *

#fonction principale
def execommande(chemin_txt, config):
    try:
        with open(chemin_txt, "r", encoding="utf-8") as fichier:
            lignes = [ligne.strip() for ligne in fichier if ligne.strip()]
        
        driver = webdriver.Chrome(service=webdriver.ChromeService('../chromedriver.exe'))
        headless_mode = config.getboolean('General', 'headless')
        pathDriver = config.get('General', 'pathToDriver')
        typeDriver = config.get('General', 'typeDriver')

        if typeDriver == 'Edge':
            options = webdriver.EdgeOptions()
            options.add_argument("--window-size=1920,1080")
            if headless_mode:
                options.add_argument("--headless=new")
            driver = webdriver.Edge(options=options, service=webdriver.EdgeService(pathDriver))
        elif typeDriver == 'Chrome':
            options = webdriver.ChromeOptions()
            options.add_argument("--window-size=1000,1080")
            if headless_mode:
                options.add_argument("--headless=new")
            driver = webdriver.Chrome(options=options, service=webdriver.ChromeService(pathDriver))
        else:
            raise Exception("Type driver not supported yet")

        #Initialisation des variables
        variables = {}
        fonctions = {}
        moduleDict = {}
        listIns = []
        res = None
        condition = []
        ext = False 
        ajtFor = []      
        newID = []
        row = []
        liste = []
        returnValue = None
        nomFonction = None
        exeWhile = None
        ajtCondWhile = []
        cmdWhile = []
        condWhile = []
        isTry = False
        isExcept = False
        cmdTry = []
        cmdExcept = []
        isWhile = []
        isFor = []
        
       #Déclaration de commandes, de fonctions, de boucles, de blocs try-except
        for ligne in lignes:
            mots = ligne.split()
            (fonctions, moduleDict, listIns, nomFonction, ajtFor, row, liste, newID,ajtCondWhile,condWhile,cmdWhile,isTry,isExcept,cmdTry,cmdExcept,isWhile,isFor) = declaration(
                isFor, isWhile,isTry, isExcept, cmdTry, cmdExcept,condWhile, cmdWhile,ajtCondWhile, liste ,row, ajtFor, newID, mots, nomFonction, fonctions, moduleDict, listIns)
            
        print(Fore.RESET)

        #Exécution des commandes
        for ins in listIns:
            
            verboseWhite(f"Execution de la commande : {ins}")
            mots = isVariable(ins, variables) 
            mots = convertype(mots)    

            if len(condition) > 0:
                ext = False
            
            variables, retourne = execute(driver, mots, moduleDict, variables, config, condition, ext, liste, row, newID,returnValue, exeWhile)
            verboseWhite(f"Commande exécutée")
            verboseWhite('-' * 50)

    #Gestion des erreurs
    except KeyboardInterrupt:
        
        erreur(f"\n⚠️  Interruption manuelle détectée. Le programme a été arrêté par l'utilisateur")
        
    except FileNotFoundError:
    
        erreur(f"\n❌ Le fichier spécifié '{chemin_txt}' est introuvable.")
    
    except pyodbc.Error as db_err:
        
        erreur(f"\n❌ Erreur de connexion à la base de données : {db_err}")
        
    except Exception as e:
        
        verboseRed(f"\n❌ Une erreur inattendue s'est produite : {e}")
        raise e
    
    finally:
        try:
            driver.close()
            verboseWhite(f"✅ Le navigateur a été fermé correctement.")
                
        except:
            
            erreur(f"⚠️  Le navigateur était déjà fermé ou n'a pas pu être fermé.")   

    return retourne