from colorama import init, Fore, Style, Back
import configparser
from selenium.webdriver.common.by import By
import importlib
from datetime import datetime, timedelta
import sys
from arguments import *
from selenium.webdriver.remote.webdriver import WebDriver
from selenium.webdriver.remote.webelement import WebElement

#Permet de remplacer les variables commençant par $
def isVariable(mots, variables):
    if isinstance(mots, str): 
        mots = mots.split()

    mots = mots[:]  

    for n, mot in enumerate(mots):
        if isinstance(mot, list):
            continue

        if mot in ["''", '""']: 
            mots[n] = 'vide'

        elif mot.startswith("$") and "[" in mot and "]" in mot:  
            nomlist = mot.split('[')[0][1:]
            indice = mot.split('[')[1][:-1]

            if indice.startswith("$"):
                indice = variables[indice[1:]]

            if nomlist in variables:
                if isinstance(variables[nomlist], list):
                    try:
                        mots[n] = variables[nomlist][int(indice)]
                    except (IndexError, ValueError):
                        erreur(f"Erreur : Indice invalide pour la liste '{nomlist}'")
                elif isinstance(variables[nomlist], dict):
                    res = variables[nomlist]
                    if isinstance(indice,str) and indice.isdigit():
                        indice = int(indice)
                    res = res[indice]
                    mots[n] = res 

        elif mot.startswith("$"):  
            motSansDollar = mot[1:]
            mots[n] = variables.get(motSansDollar, mots[n])

    return mots

#permet de convertir les éléments d'une liste en leur type approprié 
def convertype(mots):
    if len(mots) == 1: 
        mot = mots[0]
        if isinstance(mot, (int, bool, type(None))): 
            return mot
        elif isinstance(mot, str):
            if mot.lstrip('-').isdigit():  
                return int(mot)
            elif mot == "True":
                return True
            elif mot == "False":
                return False
            elif mot == "None":
                return None
            else:
                return [mot]  
        elif isinstance(mot, (WebElement,WebDriver)):
            return mot  
        else:
            return mot
    else:  
        resultat = []
        for mot in mots:
            if isinstance(mot, (int, bool, type(None))):  
                res = mot
            elif isinstance(mot, str) and mot.lstrip('-').isdigit():  
                res = int(mot)
            elif mot == "True":
                res = True
            elif mot == "False":
                res = False
            elif mot == "None":
                res = None
            elif isinstance(mot, (WebElement,WebDriver)):
                res = mot 
            else:
                res = mot  
            resultat.append(res)
        return resultat

#Affecte un résultat à une variable
def affecResult(resultat, mots, variables, aff):
    try:
        if "=" in mots:
            variable = mots[0]

            if "[" in variable and "]" in variable:  
                nom_var = variable.split('[')[0]
                indice = variable.split('[')[1][:-1]

                if nom_var not in variables:
                    variables[nom_var] = []
                    if aff:
                        verbose(f"Variable '{nom_var}' initialisée comme une liste.")

                if isinstance(variables[nom_var], list):
                    indice = int(indice)
                    while len(variables[nom_var]) <= indice:
                        variables[nom_var].append(None)  
                    variables[nom_var][indice] = resultat
                    if aff:
                        verbose(f"Variable : '{nom_var}' à l'indice [{indice}] définie avec la valeur {resultat}")
                
                elif isinstance(variables[nom_var], dict):
                    variables[nom_var][indice] = resultat
                    if aff:
                        verbose(f"Variable : '{nom_var}' avec la clé '{indice}' définie avec la valeur {resultat}")

                else:
                    if aff:
                        erreur(f"Erreur : La variable '{nom_var}' n'est ni une liste ni un dictionnaire.")

            else: 
                variables[variable] = resultat
                if aff:
                    verbose(f"Variable : '{variable}' définie avec la valeur {resultat}")

    except ValueError as ve:
        verboseRed(f"Erreur lors de l'affectation : Indice invalide pour une liste ({ve}).")
        raise ve
    except Exception as e:
        verboseRed(f"Erreur lors de l'affectation : {e}")
        raise e

#Affecte le résultat d'une commande à une variable
def affectation(config, variables, driver, mots, module):

    #permet de récupérer une donnée stockée dans un fichier de configuration INI
    if "get.data" in mots:
        verbose(f"Étape : Commande 'get.data' détectée, récupération des données à partir du fichier INI.")

        section = mots[4]
        variable = mots[6]

        verbose(f"Étape : Récupération de la section '{section}' et de la variable '{variable}'.")
        
        try:
            valeur = config.get(section, variable)

            verbose(f"Étape : Valeur récupérée : {valeur}")
            affecResult(valeur, mots, variables,aff=True)         

        except configparser.NoSectionError as e:
            verboseRed(f"Erreur : La section '{section}' n'existe pas dans le fichier INI.")
            raise e
            
        except configparser.NoOptionError:
            erreur(f"Erreur : La variable '{variable}' n'existe pas dans la section '{section}'.")
            mots = ''  

    #permet de récuperer le contenu d'un tableau sous forme de liste
    elif "get.table" in mots:
        verbose(f"Étape : Commande 'get.table' détectée, récupération des données à partir du tableau.")

        try:
            tables = driver.find_elements(By.XPATH, mots[3])
            if not tables:
                raise ValueError(f"Aucun tableau trouvé avec le XPATH spécifié : {mots[3]}")

            table = tables[0]
            rows = table.find_elements(By.TAG_NAME, "tr")

            flattened_data = []

            for row in rows:
                try:
                    cols = row.find_elements(By.TAG_NAME, "td")
                    row_data = [col.text for col in cols] 
                    flattened_data.extend(row_data)  
                except Exception as row_error:
                    erreur(f"Erreur lors du traitement d'une ligne : {row_error}")

            if isinstance(mots[0], str):
                variables[mots[0]] = flattened_data
            else:
                variables[tuple(mots[0])] = (flattened_data)

            verboseYellow(f"Données extraites : {flattened_data}")

        except ValueError as ve:
            if args.erreur:
                erreur(f"Erreur : {ve}")
        except Exception as e:
            erreur(f"Erreur inattendue lors de la récupération des données de la table : {e}")
 
    #permet de récuperer le contenu d'un tableau sous forme de liste de WebElement
    elif 'table' in mots:
        try:
            chemin = ' '.join(mots[3:])

            if chemin.startswith(("/", "//")):
                table = driver.find_elements(By.XPATH, chemin)
                method = "XPath"
            elif chemin.startswith(".") :
                table = driver.find_elements(By.CSS_SELECTOR, chemin)
                method = "CSS Selector"
            elif chemin.startswith("#"):
                chemin = ''.join(chemin[1:])
                table = driver.find_elements(By.ID, chemin)
                method = "ID"
                    
            if not table:
                raise ValueError(f"Aucune donnée trouvée avec le {method} fourni.")
            affecResult(table, mots, variables, aff=False)
            verboseYellow(f"Données extraites : {variables[mots[0]]} via {method}")
            
        except :
            table = 'vide'
            affecResult(table, mots, variables, aff=False)
            verboseYellow(f"Aucunes données extraites")

    #permet de récuperer le titre de la page
    elif 'get.title' in mots:
        try:
            res = driver.title
            verbose(f"Titre de la page récupéré : {res}")
            affecResult(res, mots, variables, aff=True)
        except Exception as e:
            erreur(f"Une erreur inattendue s'est produite lors de la récupération du titre : {e}")

    #permet d'initialiser une variable à []
    elif mots[2] == '[]':
        variables[mots[0]] = []
        verbose(f"Variable : la liste {mots[0]} définie avec la valeur {variables[mots[0]]}")

    #permet d'initialiser une variable avec une chaîne vide qui sera stocké comme 'vide'
    elif mots[2] == "''" or mots[2] == '""':
        variables[mots[0]] = 'vide'
        verbose(f"Variable : la liste {mots[0]} définie avec la valeur ''")

    #permet d'additionner 2 valeurs
    elif '+' in mots:
        add1 = mots[2] 
        add2 = mots[4]
        if isinstance(add1, int) and isinstance(add2,int):
            res = add1 + add2
            affecResult(res,mots,variables,aff=True)

    #permet de soustraire 2 valeurs
    elif '-' in mots:
        sous1 = mots[2]
        sous2 = mots[4]

        try:
            if isinstance(sous1, str) and "/" in sous1:
                sous1 = datetime.strptime(sous1, "%d/%m/%Y")
            elif isinstance(sous1, str) and "-" in sous1:
                sous1 = datetime.fromisoformat(sous1)

            if isinstance(sous2, str) and "/" in sous2:
                sous2 = datetime.strptime(sous2, "%d/%m/%Y")
            elif isinstance(sous2, str) and "-" in sous2:
                sous2 = datetime.fromisoformat(sous2)

            res = sous1 - sous2
            verbose(f"{sous1} - {sous2} = {res}")
            affecResult(res, mots, variables,aff=True)

        except ValueError as e:
            erreur(f"Erreur de conversion ou de soustraction : {e}")

    #Renvoie la valeur absolue d'une variable
    elif 'abs' in mots:
        var = mots[3]
        res = abs(var)
        affecResult(res,mots,variables,aff=True)

    #Concatène 2 ou plusieurs valeurs
    elif '&' in mots:
        if len(mots) >= 5 and mots[1] == '=':  
            var_name = mots[0]
            elements = mots[2:]
            res = ""

            for elem in elements:
                if elem != '&':
                    res += str(elem)  
            
            verbose(f"Concaténation : {res}")
            affecResult(res, mots, variables, aff=True)
        else:
            erreur("❌ Erreur : Syntaxe incorrecte pour la concaténation. Utilisation : $var = val1 & val2 & ...")

    #permet de diviser une chaîne de caractères selon un séparateur avec des paramètre comme le nombre de divisions ou l'extraction d'un élément spécifique
    elif 'split' in mots:
        try:
            string = mots[3]
            sepa = mots[4].replace("'", '').replace('"', '')  
            if sepa == 'espc':
                sepa = ' ' 

            if len(mots) > 5 and '(' in mots[5] and ')' in mots[5]:
                nbfois = int(mots[5].replace('(', '').replace(')', ''))
                index_position = 6
            else:
                nbfois = -1 
                index_position = 5

            if len(mots) > index_position and '[' in mots[index_position] and ']' in mots[index_position]:
                index = int(mots[index_position].replace('[', '').replace(']', ''))
            else:
                index = None  

           
            res_list = string.split(sepa, nbfois)

            if index is not None:
                if index < len(res_list):
                    res = res_list[index]
                else:
                    res = None
                    erreur(f"L'index [{index}] est hors limite pour le résultat du split.")
            else:
                res = res_list  

            affecResult(res, mots, variables, aff=False)
            verbose(f"Résultat du split : {res}")

        except Exception as e:
            erreur(f"❌ Erreur lors de l'exécution de split : {e}")
    
    #permet de diviser une chaîne de caractères en commençant par la droite selon un séparateur avec des paramètre comme le nombre de divisions ou l'extraction d'un élément spécifique
    elif 'rsplit' in mots:

        string = mots[3]
        sepa = mots[4].replace("'", '').replace("'",'')
        if sepa == 'espc':
            sepa = ' '
        nbfois = int(mots[5].replace('[', '').replace(']',''))
        res = string.rsplit(sepa,nbfois)
        affecResult(res, mots, variables,aff=True)

    #permet de remplacer une valeur par une autre 
    elif 'replace' in mots:
        var = mots[2]
        oldValue = mots[4]
        newValue = mots[5] 
        if newValue == 'vide':
            newValue = ""

        res = var.replace(oldValue,newValue)
        affecResult(res,mots,variables,aff=True)

    #permet de convertir un élément string en un élement date en indiquant la forme de date 
    elif 'string2date' in mots:

        try:
            var = mots[3]
            frm = ' '.join(mots[4:])

            if isinstance(var, str):
                var = datetime.strptime(var, frm)

            affecResult(var, mots, variables,aff=True)
            
        except ValueError as e:
            erreur(f"Erreur de formatage ou de conversion : {e}")
        except Exception as e:
           erreur(f"Une erreur inattendue s'est produite : {e}")

    #renvoie la date à l'instant
    elif 'dateNow' in mots:
        variables[mots[0]] = datetime.now()
        verbose(f"Variable : {mots[0]} définie avec la valeur {variables[mots[0]]}")

    #renvoie la date d'aujourd'hui
    elif 'dateToday' in mots:
        variables[mots[0]] = datetime.today()
        verbose(f"Variable : {mots[0]} définie avec la valeur {variables[mots[0]]}")

    #permet d'ajouter(ou de soustraire) des jours à une date
    elif 'add_days' in mots:
        try:
            variable = mots[0]
            var = mots[3]
            indx = int(mots[4]) 


            if isinstance(var, str):
                try:
                    for date_format in ["%Y-%m-%d", "%d/%m/%Y", "%d-%m-%Y"]:
                        try:
                            var = datetime.strptime(var, date_format)
                            break
                        except ValueError:
                            continue
                    else:
                        raise ValueError(f"Impossible de convertir la chaîne '{var}' avec les formats attendus.")
                except ValueError as e:
                    raise ValueError(f"Erreur de conversion : {e}")

            if isinstance(var, datetime):
                res = var + timedelta(days=indx)
            elif isinstance(var, timedelta):
                res = var + timedelta(days=indx) 

            affecResult(res,mots,variables,aff=True)

        except ValueError as e:
            erreur(f"Erreur de formatage ou de conversion : {e}")
        except Exception as e:
            erreur(f"Une erreur inattendue s'est produite : {e}")

    #permet d'ajouter(ou de soustraire) des heures à une date
    elif 'add_hour' in mots:
        try:
            var = mots[3] 
            indx = int(mots[4]) 
            frm = ' '.join(mots[5:])  

            if isinstance(var, str):
                try:
                    if " " in var:
                        var = datetime.strptime(var, "%Y-%m-%d %H:%M:%S")
                    else:
                        var = datetime.strptime(var, "%Y-%m-%d")
                except ValueError as e:
                    raise ValueError(f"Impossible de convertir la chaîne '{var}' avec le format attendu.")

            res = var + timedelta(hours=indx)
            affecResult(res,mots,variables,aff=True)

        except ValueError as e:
            erreur(f"Erreur de formatage ou de conversion : {e}")
        except Exception as e:
            erreur(f"Une erreur inattendue s'est produite : {e}")

    #permetde convertir un élément date en un élément string en indiquant la forme
    elif 'date2string' in mots:
        if len(mots) < 3:
            erreur(f"Erreur dans la commande 'date2string'. La syntaxe correcte est : date2string <date> <format>")
            return
        
        var = mots[3]
        frm = ' '.join(mots[4:])
    
        try:
            res = var.strftime(frm)
            affecResult(res,mots,variables,aff=True)
        except ValueError as e:
            erreur(f"Erreur dans le format de date '{frm}': {e}")
            return

    #permet de convertir un objet timedelta en seconde
    elif 'total_seconds' in mots:
        try:
            var = mots[3]
            verbose(f"Calcul de total_seconds pour : {var}")

            if isinstance(var, timedelta):
                res = var.total_seconds()
                verbose(f"Résultat (timedelta) : {res} secondes")
            else:
                raise TypeError(f"La variable '{mots[3]}' n'est ni un timedelta. Valeur actuelle : {var}")

            affecResult(res,mots,variables,aff=True)

        except (TypeError, ValueError) as e:
            erreur(f"Erreur : {e}")
        except Exception as e:
            erreur(f"Une erreur inattendue s'est produite : {e}")
   
    #permet de récupérer un attribut spécifique d'un webElement ou webDriver en utilisant un chemin
    elif 'find_element' in mots and 'get_attribute' in mots:
        row = mots[2]
        if row == 'driver':
            row = driver
        chemin = mots[4]
        attr = mots[6]

        try:
            if isinstance(row,str):
                verboseRed("C'est une chaîne")
            if isinstance(row, (WebDriver, WebElement)):
                element = row.find_element(By.XPATH, chemin)
                res = element.get_attribute(attr) or 'vide'
                affecResult(res, mots, variables, aff=False)
                verbose(f"Valeur de l'attribut '{attr}' trouvée : {res}")
            else:
                erreur(f"⚠ Erreur : '{row}' n'est pas un WebDriver ou un WebElement valide.")
        except Exception as e:
            res = 'vide'
            affecResult(res, mots, variables, aff=False)
            verboseRed(f"Aucune valeur de l'attribut '{attr}' trouvée.")
    
    #permet de récupérer l'attribut d'un élément web
    elif 'get_attribute' in mots:
        var = mots[2]
        attr = mots[4]

        try:
            if isinstance(var, (WebDriver, WebElement)):
                res = var.get_attribute(attr)
                affecResult(res, mots, variables, aff=False)
                verbose(f"Valeur de l'attribut '{attr}' trouvée : {res}")
            else:
                erreur(f"⚠ Erreur : '{var}' n'est pas un WebDriver ou un WebElement valide.")
        except Exception as e:
            res = 'vide'
            affecResult(res, mots, variables, aff=False)
            verboseRed(f"Aucune valeur de l'attribut '{attr}' trouvée.")

    #permet de trouver un élément sur une page web en utilisant un XPath
    elif 'get_element' in mots:
        element = mots[2]
        chemin = mots[4]
        res = element.find_element(By.XPATH, chemin)
        affecResult(res, mots, variables,aff=True)

    ##permet de trouver plusieurs éléments sur une page web en utilisant un XPath
    elif 'get_elements' in mots:
        element = mots[2]
        chemin = mots[4]
        res = element.find_elements(By.XPATH, chemin)
        affecResult(res, mots, variables,aff=True)

    #permet de récupérer une propriété CSS d'un élément sur une page
    elif 'css_property' in mots:
        element = mots[2]
        attr = mots[4]

        res = element.value_of_css_property(attr)
        affecResult(res,mots,variables,aff=True)

    #permet d'affecter une valeur à une position spécifique d'une liste ou à une clé spécifique d'un dictionnaire
    elif '[' in mots[0] and ']' in mots[0]:
        nomListe = mots[0].split('[')[0]  
        indice = mots[0].split('[')[1][:-1] 

        if indice.startswith('$'):
            indice = variables[indice[1:]]

        try:
            if nomListe in variables:
                if isinstance(variables[nomListe], list):
                    indice = int(indice)  

                    while len(variables[nomListe]) <= indice:
                        variables[nomListe].append(None) 

                    tmp =  mots[2]  
                    affecResult(tmp,mots,variables,aff=False)

                    verbose(f"Variable : {nomListe} à l'indice {indice} définie avec la valeur {tmp}")

                elif isinstance(variables[nomListe], dict):
                    tmp = variables.get(mots[2], mots[2])  
                    affecResult(tmp,mots,variables,aff=False)  

                    verbose(f"Variable : {nomListe} avec la clé {indice} définie avec la valeur {tmp}")
        except Exception as e:
            erreur(f"Erreur lors de l'affectation : {e}")
            sys.exit()

    #permet d'initialiser un dictionnaire vide ou avec des valeurs dedans
    elif isinstance(" ".join(map(str, mots[2:])), str) and '{' in " ".join(map(str, mots[2:])) and '}' in " ".join(map(str, mots[2:])):

        dic_string = " ".join(mots[2:]).strip()

        if dic_string.count("{") == dic_string.count("}"):
            import re
            dic_string = re.sub(r'(:\s*)0(\d+)', r'\1\2', dic_string)  

            try:
                dic = eval(dic_string)  
                if isinstance(dic, dict):  
                    affecResult(dic,mots,variables,aff=True) 
                else:
                    erreur("Erreur : Ce n'est pas un dictionnaire valide.")
            except Exception as e:
                erreur(f"Erreur lors de l'évaluation du dictionnaire : {e}")
        else:
            erreur("Erreur : Les délimiteurs { et } ne sont pas correctement équilibrés.")
    
    #permet d'affecter un dictionnaire dans une liste
    elif (isinstance(variables.get(mots[0]), (list, dict)) and isinstance(variables.get(mots[2]), (list, dict)) and mots[1] == '='):
        list_name = mots[0][0:]
        dict_name = mots[2][0:]

        if dict_name in variables and isinstance(variables[dict_name], dict):
            variables[list_name] = variables[dict_name]
            verbose(f"Variable : La liste {list_name} a été définie avec les valeurs du dictionnaire {dict_name}.")
        else:
            erreur(f"Erreur : {dict_name} n'est pas un dictionnaire valide.")

    #permet d'initialiser une liste avec des valeurs dedans
    elif isinstance(mots[2],str) and " ".join(mots[2:]).startswith("[") and " ".join(mots[2:]).endswith("]"):  
        try:
            liste_str = " ".join(mots[2:]) 
            liste = eval(liste_str)  

            if isinstance(liste, list):  
                affecResult(liste,mots,variables,aff=True)
            else:
                erreur(f"Erreur : {mots[0]} n'est pas une liste valide.")

        except Exception as e:
            erreur(f"Erreur lors de la création de la liste {mots[0]} : {e}")
    
    elif mots[2] in module:
        if mots[3] == 'config':
            parameters = [config]
        else:
            parameters = mots[3:] if len(mots) >= 4 else []
        res = module[mots[2]](*parameters)
        variables[mots[0]] = res
        verbose(f"Résultat : {variables[mots[0]]}")
        verbose(f"Variable : {mots[0]} définie avec la valeur {variables[mots[0]]}")
 
    #permet de supprimer les espaces inutiles ou autres caractères inutiles
    elif 'strip' in mots:
        var_name = mots[3]
        var = str(var_name) 
        res = var.strip()
        affecResult(res, mots, variables, aff=True)
        verbose(f"Valeur après strip : {res}")

    #affecte toutes autres commandes comme chaine de caractère
    else: 
        tmp = mots[2:]
        res = convertype(tmp)
        if isinstance(res,list):
            res = res[0]
        affecResult(res,mots,variables,aff=True)

    return variables 

#Ajoute les commandes, les fonctions, les boucles, les blocs try-except dans la liste des commandes qui vont être exécuter
def declaration(isFor, isWhile,isTry, isExcept, cmdTry, cmdExcept,condWhile, cmdWhile, ajtCondWhile, liste, row, ajtFor, newID, mots, nomFuncInProcess = None, fonctions = {}, moduleDict = {}, listIns = []):
   
    cle = mots[0]

    #indique qu'on commence la définition d'une fonction
    if cle == "def":
        nom = mots[1]
        parametre = mots[2].replace('(','').replace(')','')
        fonctions[nom] = [parametre]
        verboseBlue(f"Fonction {nom} ajoutée avec le paramètre : {parametre}.")
        verboseBlue(f"Début de la définition de la fonction : {nom}")
        nomFuncInProcess = nom

    #indique qu'on a finis la définiton d'une fonction
    elif cle == "end":
        verboseBlue(f"Fin de la définition de la fonction ")
        verboseBlue(f"Fonction : {nomFuncInProcess}({fonctions[nomFuncInProcess][0]}) définie avec les commandes suivantes : {fonctions[nomFuncInProcess][1:]}")
        nomFuncInProcess = None

    #ajoute les commandes tant qu'on est dans une fonction
    elif nomFuncInProcess is not None:
        fonctions[nomFuncInProcess].append(mots)
        verboseBlue(f"Commande ajoutée : {' '.join(mots)}")

    #ajoute dans la liste des commandes les commandes avec une fonction 
    elif cle in fonctions:
        parametre = fonctions[cle][0]
        argument = mots[1].replace('(','').replace(')','')

        for cmd in fonctions[cle][1:]:
            commande_executee = [argument if mot == parametre else mot for mot in cmd]
            verboseBlue(f"Étape : Commande ajoutée: {commande_executee}")
            listIns.append(commande_executee)

        verboseBlue(f"Fonction : {cle} exécutée avec succès")

    #importe un fichier python
    elif mots[0] == "from" and mots[2] == "import":
        source = mots[1]
        fonction = mots[3]

        try:
            module = importlib.import_module(source)
            func = getattr(module, fonction)
            moduleDict[fonction] = func

            verboseBlue(f"{moduleDict[fonction]}")
            verboseBlue(f"Fonction '{fonction}' importée depuis '{source}'")

        except ModuleNotFoundError:
            erreur(f"Erreur : Le module '{source}' n'a pas été trouvé.")
        except AttributeError:
            erreur(f"Erreur : La fonction '{fonction}' n'existe pas dans le module '{source}'.")
        except Exception as e:
            erreur(f"Erreur inattendue : {e}")

    #importe un fichier texte
    elif mots[0] == "import" and mots[1] == "cmd":
        if len(mots) < 3:
            erreur("Erreur : Syntaxe incorrecte. Utilisation : import cmd <chemin_du_fichier>")
        else:
            chemin_txt = mots[2]

            try:
                with open(chemin_txt, "r", encoding="utf-8") as fichier:
                    lignes = [ligne.strip() for ligne in fichier if ligne.strip()]

                for ligne in lignes:
                    mots_ligne = ligne.split()
                    fonctions, moduleDict, listIns, nomFuncInProcess = declaration(
                        mots_ligne, nomFuncInProcess, fonctions, moduleDict, listIns
                    )

                verboseBlue(f"Importation réussie depuis '{chemin_txt}'")

            except FileNotFoundError:
                erreur(f"❌ Le fichier '{chemin_txt}' est introuvable.")
            except Exception as e:
                erreur(f"❌ Une erreur s'est produite lors de l'importation : {e}")

    #indique le début de la définition d'une boucle
    elif cle == "while":
        cmdWhile.append([])  
        ajtCondWhile.append(True)  
        condWhile.append(mots[1:])  
        isWhile.append(False)
        verboseBlue(f"Début d'une boucle while avec la condition : {condWhile[-1]}")

    #indique la fin de la boucle while et ajoute une commande while avec les paramétre nécessaire dans la liste des commandes pour l'exécution dans la fonction exécute
    elif cle == "endwhile":
        res = ['while', condWhile[-1], ':', cmdWhile[-1]] 
        if len(cmdWhile) > 1:
            cmdWhile[-2].append(res) 
        else:
            listIns.append(res)  
        verboseBlue(f"Fin de la boucle while avec le bloc : {res}")
        ajtCondWhile.pop()
        isWhile.pop()
        condWhile.pop()
        cmdWhile.pop()

    #ajoute les commandes de la boucle while
    elif len(ajtCondWhile) >= 1 and len(isWhile) >= 1 and ajtCondWhile[-1] and not isWhile[-1]:
        verboseBlue(f"Ajout d'une commande à la boucle while {condWhile[-1]} : {mots}")
        isWhile[-1] = True  
        whileCmd = []
        (fonctions, moduleDict, whileCmd, nomFuncInProcess, ajtFor, row, liste, 
        newID, ajtCondWhile, condWhile, cmdWhile, isTry, isExcept, cmdTry, 
        cmdExcept, isWhile, isFor) = declaration(
            isFor, isWhile, isTry, isExcept, cmdTry, cmdExcept, condWhile, 
            cmdWhile, ajtCondWhile, liste, row, ajtFor, newID, mots, 
            nomFuncInProcess, fonctions, moduleDict, whileCmd)
        if whileCmd:
            cmdWhile[-1].append(whileCmd[0])
        isWhile[-1] = False  

    #indique le début de la définiton d'une boucle for
    elif cle == "for":

        verboseBlue("Début de la boucle for")
        newID.append([])
        ajtFor.append(True)
        isFor.append(False)
        pos_in = mots.index('in')

        row.append(mots[1:pos_in])
        liste.append(mots[pos_in + 1:])

        if not row[-1] or not liste[-1]:
            erreur("❌ Erreur : 'for' doit être suivi de deux arguments (variable et liste).")
    
    #indique la fin de la boucle for et ajoute une commande loop avec les paramètres nécessaire dans la liste des commandes pour pour l'éxecution dans la fonction exécute
    elif cle == "endfor":
        if not ajtFor[-1]:
            erreur("❌ Erreur : 'endfor' sans 'for' correspondant.")
        verboseBlue(f"Fin de la boucle : {row[-1]} in {liste[-1]} : {newID[-1]}")

        if newID:
            res = ["loop", row[-1], "in", liste[-1], newID[-1]]
            if len(ajtFor) > 1:
                newID[-2].append(res)
            else:
                listIns.append(res)
        else:
            erreur("⚠ Attention : Aucune instruction enregistrée dans la boucle.")

        ajtFor.pop()
        newID.pop()
        liste.pop()
        row.pop()

    #ajoute les commandes de la boucle for
    elif len(ajtFor) >= 1 and len(isFor) >= 1 and ajtFor[-1] and not isFor[-1]:
        verboseBlue(f"Ajout d'une instruction dans la boucle for : {mots}")
        isFor[-1] = True
        forCmd = []
        (fonctions, moduleDict, forCmd, nomFuncInProcess, ajtFor, row, liste, newID,ajtCondWhile,condWhile,cmdWhile,isTry,isExcept,cmdTry,cmdExcept,isWhile, isFor) = declaration(
            isFor, isWhile,isTry, isExcept, cmdTry, cmdExcept,condWhile, cmdWhile, ajtCondWhile, liste, row, ajtFor, newID, mots, nomFuncInProcess, fonctions , moduleDict,forCmd)
        if forCmd:
            newID[-1].append(forCmd[0])
        isFor[-1] = False

    #indique le début du bloc try
    elif cle == "try":
        isTry = True
        cmdTry = [] 
        verboseBlue(f"Début du bloc 'try'")

    #indique la fin du bloc except
    elif cle == "except":
        isTry = False
        isExcept = True
        cmdExcept = []  
        verboseBlue(f"Début du bloc 'except'")

    #indique la fin du bloc try-except et ajoute une commande try-except dans la liste des commandes
    elif 'endTryExcept' in mots:
        res = ['try', cmdTry, 'except', cmdExcept]
        listIns.append(res)
        verboseBlue(f"Fin du bloc 'try-except' avec les commandes : {res}")
        cmdTry = []
        cmdExcept = []
        isTry = False
        isExcept = False

    #ajoute les commandes du bloc try 
    elif isTry:
        cmdTry.append(mots)
        verboseBlue(f"Ajout de la commande dans 'try' : {mots}")

    #ajoute les commandes du bloc except
    elif isExcept:
        cmdExcept.append(mots)
        verboseBlue(f"Ajout de la commande dans 'except' : {mots}")

    #ajoute tous les autres commandes dans la liste des commandes
    else:
        if not isFor or isWhile:
            verboseBlue(f"Ajout de la commande : {mots}")
        listIns.append(mots)
        if not isFor or isWhile:
            verboseBlue(f"Étape : Commande ajoutée.")  

    return (fonctions, moduleDict, listIns, nomFuncInProcess, ajtFor, row, liste, newID,ajtCondWhile,condWhile,cmdWhile,isTry,isExcept,cmdTry,cmdExcept,isWhile, isFor)